package bfv.desktop;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Path2D;
import java.text.DecimalFormat;

/**
 * Created by IntelliJ IDEA.
 * User: Alistair
 * Date: 02/05/2011
 * Time: 9:21:42 PM
 * To change this template use File | Settings | File Templates.
 */
public class AltPanel extends JPanel {

    private double[] altData;
    private double[] filteredAltData;

    private double lastAlt;

    private int i = 0;
    private int size = 400;
    private int xStep = 1;
    private int xBorder = 0;


    private boolean dataFull;
    private DecimalFormat df1 = new DecimalFormat("0.0");
    private DecimalFormat df0 = new DecimalFormat("0");
    private DecimalFormat df;

    double altDamp = 0.05;



    public AltPanel() {
        super();

        altData = new double[size];
        filteredAltData = new double[size];
        altDamp = Double.parseDouble(BFVDesktop.properties.getProperty("altDamp"));

    }

    public void addData(double alt){

        altData[i] = alt;

        filteredAltData[i] = lastAlt * (1.0-altDamp) + alt * altDamp;

        lastAlt = filteredAltData[i];


        i++;
        if(i == size){
            i = 0;
            dataFull = true;
        }


    }

    public double getLastAlt() {
        return lastAlt;
    }

    public void fillData(double alt){

        for (int j = 0; j < altData.length; j++) {
            altData[j] = alt;

        }

        for (int j = 0; j < filteredAltData.length; j++) {
            filteredAltData[j] = alt;

        }
        lastAlt = alt;

        i = 0;


    }


    public void paintComponent(Graphics g){
        Graphics2D g2 = (Graphics2D)g;



        double min = getMinValue(altData, new double[]{});
        double max = getMaxValue(altData, new double[]{});

        double currentAlt = filteredAltData[i];
        double scale = (max - min);
        if(scale < 0.5){
            scale = 0.5;
        }


        //alt (raw)


        Path2D.Double altShape = new Path2D.Double();
        int xAlt = xBorder;
        altShape.moveTo(xBorder,110.0);
        for(int p = i; p > i-size + 1; p--){
            int c = p-1;
            if(c < 0){
                c = c + size;

            }

            double yAlt = 110.0+(currentAlt - altData[c])*100/scale;
            altShape.lineTo(xAlt, yAlt);
            xAlt+=xStep;
        }
        altShape.lineTo(xAlt,10);
        altShape.lineTo(xBorder,10);
        altShape.closePath();

        g2.setColor(Color.white);
        g2.fill(altShape);


       //alt


       Path2D.Double filteredAltShape = new Path2D.Double();
       xAlt = xBorder;
        filteredAltShape.moveTo(xBorder,110.0);
       for(int p = i; p > i-size + 1; p--){
           int c = p-1;
           if(c < 0){
               c = c + size;

           }

           double yAlt = 110.0+(currentAlt - filteredAltData[c])*100/scale;
           filteredAltShape.lineTo(xAlt, yAlt);
           xAlt+=xStep;
       }

       g2.setColor(Color.blue);
       g2.draw(filteredAltShape);



        g2.setColor(Color.black);
        g2.drawRect(xBorder,10,xStep*size,200);
       
//        System.out.println(mag);
        g2.setColor(Color.black);

        double tick = 0.2;
        df = df1;

        if  (scale > 1.0){
            tick = 1.0;
            df = df0;
        }

        double yScaleStart = min - scale - (min%tick);
        
        for(double y = yScaleStart; y < max + scale; y += tick){

            int yPix = (int)((currentAlt - y) * 100.0/scale);
            if(yPix > -100 && yPix < 100){
                g2.drawLine(xBorder,110 + yPix , xBorder + 5, 110 + yPix);
                g2.drawString(df.format(y), xBorder + 10, 110+yPix + 5);
            }


        }

    }

    public static double getMaxValue(double[] val, double[] val2){
        double max = Double.MIN_VALUE;
        for (int i = 0; i < val.length; i++) {
            double v = val[i];
            if(v > max){
                max = v;
            }
        }
        for (int i = 0; i < val2.length; i++) {
            double v = val2[i];
            if(v > max){
               max = v;
           }
        }
        return max;

    }

    public static double getMinValue(double[] val, double val2[]){
        double min = Double.MAX_VALUE;
        for (int i = 0; i < val.length; i++) {
            double v = val[i];
            if(v < min){
                min = v;
            }
        }

        for (int i = 0; i < val2.length; i++) {
            double v = val2[i];
            if(v < min){
                min = v;
            }
        }
        return min;

    }
}

