package bfv.desktop;

import bfv.hardware.HardwareCommand;
import bfv.hardware.HardwareSettingPanel;
import bfv.jssc.SerialInputStream;
import bfv.jssc.SerialOutputStream;
import bfv.util.*;
import jssc.*;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.TimeZone;

/**
 * Created by IntelliJ IDEA.
 * User: Alistair
 * Date: 27/04/2011
 * Time: 7:43:46 PM
 * To change this template use File | Settings | File Templates.
 */
public class BFVDesktop extends JPanel implements ActionListener, WindowListener, Runnable, PopupMenuListener, ItemListener {


    public static JFrame frame;

    public static Properties properties;

    public static final String propertiesFileName = "BFVDesktop.properties";

    private JTextArea text;
    private JTextField input;


    private JComboBox serialPorts;


    public JLabel altText;
    public JLabel tempText;
    public JLabel batText;
    public JCheckBox append;

    public AltPanel altPanel;


    public double alt = -10000000.0;

    private double altData[] = new double[100];

    private int alti;

    public Statistics altStats = new Statistics(altData);
    public boolean firstAlt = false;

    public HardwareSettingPanel hardwareSettingPanel;

    private JPanel leftPane;

    private JTextField baudField;

    private JButton serial;

    //textArea
    public TextAreaOutputStream textStream;


    //SerialPortStuff

    private SerialPort serialPort;

    //output Tx
    public PrintWriter out;
    public OutputStream outputStream;

    //input Rx
    public InputStream inStream;

    public static final String hardwareVersionText = "Hardware Version: ";
    public JLabel hardwareVersion;

    private JComboBox commands;
    private JButton issueCommand;
    private JTextField commandParams;


    private PrintWriter printWriterMTK;
    private PrintWriter printWriterCSV;


    public BFVDesktop() {
        super();
        properties = new Properties();
        try {
            properties.load(new FileInputStream(propertiesFileName));
        } catch (IOException e) {
            e.printStackTrace();
        }

        this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

        leftPane = new JPanel();
        leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.Y_AXIS));
        leftPane.setBorder(new TitledBorder("Send to Vario"));
        leftPane.setAlignmentX(Component.LEFT_ALIGNMENT);
        leftPane.setAlignmentY(Component.TOP_ALIGNMENT);


        JPanel serialPane = new JPanel();
        serialPane.setLayout(new BoxLayout(serialPane, BoxLayout.X_AXIS));
        serialPane.setBorder(new TitledBorder("Serial Port"));
        serialPane.setAlignmentX(0.0f);

        String[] ports = SerialPortList.getPortNames();
        serialPorts = new JComboBox(ports);
        serialPorts.setPreferredSize(new Dimension(100, 24));
        serialPorts.setMaximumSize(new Dimension(100, 24));
        serialPorts.setMinimumSize(new Dimension(100, 24));

        serialPorts.addPopupMenuListener(this);

        String lastPortName = properties.get("comPort").toString();
        serialPorts.setSelectedItem(lastPortName);
        serialPane.add(serialPorts);

        //serialPane.add(Box.createHorizontalGlue());

        serialPane.add(new JLabel("Baud:"));

        baudField = new JTextField(10);
        baudField.setMaximumSize(new Dimension(60, 24));
        baudField.setPreferredSize(new Dimension(60, 24));
        baudField.setMinimumSize(new Dimension(60, 24));
        baudField.setText(properties.getProperty("baudRate"));
        serialPane.add(baudField);


        serialPane.add(Box.createHorizontalGlue());

        serial = new JButton("Connect");
        serial.setActionCommand("serial");
        serial.addActionListener(this);
        serialPane.add(serial);


        //leftPane.add(Box.createRigidArea(new Dimension(100, 5)));


        JPanel consolePane = new JPanel();
        consolePane.setLayout(new BoxLayout(consolePane, BoxLayout.X_AXIS));
        consolePane.setBorder(new TitledBorder("Raw Tx:"));
        consolePane.setAlignmentX(0.0f);


        input = new JTextField(32);
        input.setPreferredSize(new Dimension(400, 24));
        input.setMaximumSize(new Dimension(400, 24));
        //input.setMinimumSize(new Dimension(4000, 20));
        input.setActionCommand("input");
        input.addActionListener(this);

        consolePane.add(input);


        hardwareSettingPanel = new HardwareSettingPanel(this);
        hardwareSettingPanel.setAlignmentX(0.0f);


        JPanel commandsPane = new JPanel();
        commandsPane.setLayout(new BoxLayout(commandsPane, BoxLayout.Y_AXIS));
        commandsPane.setBorder(new TitledBorder("Hardware Commands"));
        //commandsPane.setAlignmentX(0.0f);

        commands = new JComboBox(hardwareSettingPanel.getHardwareCommands().getCommands().toArray());
        commands.setMaximumSize(new Dimension(300, 24));
        commands.setPreferredSize(new Dimension(300, 24));
        commands.addItemListener(this);
        //commands.setMinimumSize(new Dimension(200, 24));

        issueCommand = new JButton("Execute");
        issueCommand.setActionCommand("issueCommand");
        issueCommand.addActionListener(this);

        JPanel topCommandsPane = new JPanel();
        topCommandsPane.setLayout(new BoxLayout(topCommandsPane, BoxLayout.X_AXIS));

        topCommandsPane.add(commands);
        topCommandsPane.add(Box.createHorizontalGlue());
        topCommandsPane.add(issueCommand);
        topCommandsPane.setAlignmentX(0.0f);

        commandParams = new JTextField(0);
        commandParams.setToolTipText("Enter parameters for some commands > {params}");
        commandParams.setMaximumSize(new Dimension(300, 24));
        commandParams.setPreferredSize(new Dimension(300, 24));
        //commandParams.setMinimumSize(new Dimension(200, 24));
        commandParams.setText("");
        commandParams.setAlignmentX(0.0f);

        commandsPane.add(topCommandsPane);
        commandsPane.add(commandParams);
        hardwareCommandParams();

        leftPane.add(serialPane);
        leftPane.add(consolePane);
        leftPane.add(commandsPane);
        leftPane.add(hardwareSettingPanel);
        this.add(leftPane);


        JPanel rightPane = new JPanel();
        rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.Y_AXIS));
        rightPane.setAlignmentX(Component.RIGHT_ALIGNMENT);
        rightPane.setBorder(new TitledBorder("Receive from Vario"));
        rightPane.setAlignmentY(Component.TOP_ALIGNMENT);

        hardwareVersion = new JLabel(hardwareVersionText);
        hardwareVersion.setAlignmentX(0.0f);
        rightPane.add(hardwareVersion);

        altText = new JLabel("Alt:");
        altText.setAlignmentX(0.0f);
        rightPane.add(altText);

        tempText = new JLabel("Temp:");
        tempText.setAlignmentX(0.0f);
        rightPane.add(tempText);

        batText = new JLabel("Bat:");
        batText.setAlignmentX(0.0f);
        rightPane.add(batText);


        JPanel prsPane = new JPanel();
        prsPane.setLayout(new BoxLayout(prsPane, BoxLayout.Y_AXIS));
        prsPane.setBorder(new TitledBorder("Pressure Stream to Altitude (when in outputMode = 0)"));
        prsPane.setAlignmentX(0.0f);

        altPanel = new AltPanel();
        altPanel.setPreferredSize(new Dimension(401, 211));
        altPanel.setAlignmentX(0.0f);
        prsPane.add(altPanel);

        rightPane.add(prsPane);

//        JLabel rx = new JLabel("Raw Rx:");
//        rx.setAlignmentX(0.0f);
//        rightPane.add(rx);

        JPanel rxPane = new JPanel();
        rxPane.setLayout(new BoxLayout(rxPane, BoxLayout.Y_AXIS));
        rxPane.setBorder(new TitledBorder("Raw Rx:"));
        rxPane.setAlignmentY(1.0f);

        text = new JTextArea(36, 35);
        JScrollPane areaScrollPane = new JScrollPane(text);
        areaScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        //areaScrollPane.setPreferredSize(new Dimension(500, 200));
        areaScrollPane.setAlignmentX(0.0f);

        text.setEditable(false);
        textStream = new TextAreaOutputStream(text);
        rxPane.add(areaScrollPane);

        JPanel textOptions = new JPanel();
        textOptions.setLayout(new BoxLayout(textOptions, BoxLayout.X_AXIS));
        textOptions.setAlignmentX(0.0f);

        append = new JCheckBox("Stream Text");
        append.setSelected(true);
        textOptions.add(append);

        textOptions.add(Box.createHorizontalGlue());

        JButton clear = new JButton("Clear");
        clear.setActionCommand("clear");
        clear.addActionListener(this);
        textOptions.add(clear);

        rxPane.add(textOptions);

        rightPane.add(rxPane);


        this.add(rightPane);


    }

    public void hardwareCommandParams() {
        HardwareCommand cmd = (HardwareCommand) commands.getSelectedItem();
        if (cmd.getParams() != null) {

            commandParams.setText(cmd.getParams());
            commandParams.setEnabled(true);
            commandParams.setBackground(Color.WHITE);
        } else {
            commandParams.setText("");
            commandParams.setEnabled(false);
            commandParams.setBackground(Color.GRAY);
        }

    }

    public void addComponentLeft(JComponent component) {
        leftPane.add(component, 0);
    }

    public void addAltData(double alt) {
        altData[alti] = alt;

        alti++;
        if (alti >= altData.length) {
            alti = 0;
        }

    }

    public void sendOutput(String line, boolean echo) {
        if (echo) {
            textStream.println(line);
        }
        if (out != null) {
            out.write(line);
            out.write("\r\n");
            out.flush();

        }

    }

    public void getParametersFromDevice() {

        sendOutput("$BST*", true);

    }

    public TextAreaOutputStream getTextStream() {
        return textStream;
    }

    public void setOut(PrintWriter out) {
        this.out = out;
    }

    public void setOutputStream(OutputStream outputStream) {
        this.outputStream = outputStream;
    }

    public void setInStream(InputStream inStream) {
        this.inStream = inStream;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("serial")) {
            if (serial.getText().equals("Connect")) {
                firstAlt = false;
                String port = (String) (serialPorts.getSelectedItem());

                if (serialPort != null && serialPort.isOpened()) {
                    try {
                        serialPort.closePort();
                        serialPort = null;
                    } catch (SerialPortException e1) {
                        e1.printStackTrace();
                        return;
                    }
                }
                serialPort = new SerialPort(port);


                int baudRate = Integer.parseInt(baudField.getText());

                //baudRate = 115200;
                int dataBits = Integer.parseInt(properties.getProperty("dataBits"));
                int stopBits = Integer.parseInt(properties.getProperty("stopBits"));
                int parity = Integer.parseInt(properties.getProperty("parity"));


                try {
                    serialPort.openPort();//Open port
                    properties.setProperty("comPort", port);
                    serialPort.setParams(baudRate, dataBits, stopBits, parity);
                    properties.setProperty("baudRate", baudRate + "");

                    inStream = new SerialInputStream(serialPort);
                    outputStream = new SerialOutputStream(serialPort);
                    out = new PrintWriter(outputStream);

                    new Thread(this).start();
                    getParametersFromDevice();

                } catch (SerialPortException ex) {
                    JOptionPane.showMessageDialog(frame, ex.toString());
                    System.out.println(ex);
                }
                serial.setText("Disconnect");

            } else if (serial.getText().equals("Disconnect")) {
                try {

                    serialPort.closePort();

                } catch (SerialPortException e1) {
                    JOptionPane.showMessageDialog(frame, e1.toString());
                    e1.printStackTrace();
                }
                serial.setText("Connect");

            }


        } else if (e.getActionCommand().equals("input")) {
            String in = input.getText().trim();
            input.setText("");

            printOutputLine(in);


        } else if (e.getActionCommand().equals("clear")) {

            text.setText("");


        }


        if (e.getActionCommand().equals("issueCommand")) {
            HardwareCommand cmd = (HardwareCommand) commands.getSelectedItem();
            if (commandParams.isEnabled()) {
                cmd.setParams(commandParams.getText());
            }
            hardwareSettingPanel.getHardwareCommands().sendCommand(cmd);

        }


    }

    public void printOutputLine(String in) {

        textStream.println(in);
        if (in.endsWith("<") && in.length() > 1) {
            out.write(in.substring(0, in.length() - 1) + "\n");
        } else if (in.startsWith("#")) {
            try {
                String s = in.substring(1, in.length());
                textStream.println(s);
                byte[] bytes = hexStringToByteArray(s);
                for (int i = 0; i < bytes.length; i++) {
                    byte aByte = bytes[i];
                    textStream.println(aByte + " " + String.format("%02X", aByte));
                }
                outputStream.write(bytes);
                outputStream.flush();
            } catch (IOException e1) {
                e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        } else {
            in = in.replaceAll("<CR>", "\r");
            in = in.replaceAll("<LF>", "\n");
            if (out != null) {
                out.write(in);
            }

        }

        out.flush();

    }


    public void run() {

        BufferedReader reader = new BufferedReader(new InputStreamReader(inStream));

        String lineRead = null;


        double qnh = Double.parseDouble(properties.getProperty("qnh"));


        DecimalFormat df1 = new DecimalFormat("0.0");
        DecimalFormat df = new DecimalFormat("+0.0;-0.0");
        DecimalFormat df2 = new DecimalFormat("0.00");

        DecimalFormat df4 = new DecimalFormat("0.0000");


//        PrintStream out = null;
//
//
//        try {
//            out = new PrintStream(new FileOutputStream("out.csv"));
//        } catch (IOException e) {
//            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
//        }


        long time = 0;

        double pressureDiff = 0.0;


        try {


            while ((lineRead = reader.readLine()) != null) {

                if(lineRead.startsWith("$PMTK")){
                    PMTKParser(lineRead);
                }
                String[] split = lineRead.split(" ");


                if (append.isSelected()) {
                    textStream.println(lineRead);
//                    if (split.length == 0) {
//                        textStream.println(lineRead);
//                    } else if (!split[0].equals("BAT")) {
//                        textStream.println(lineRead);
//                    }
                }


                if (split.length > 0) {
                    if (split[0].equals("PRS")) {


                        try {
                            int pressure = Integer.parseInt(split[1], 16);

                            double rawAlt = 44330.0 * (1 - Math.pow((pressure / qnh), 0.190295));


                            alt = rawAlt;

                            if (!firstAlt) {
                                altPanel.fillData(alt);
                                firstAlt = true;
                            } else {
                                altPanel.addData(rawAlt);
                                this.addAltData(rawAlt);
                            }


                            altText.setText("Alt: " + df2.format(altPanel.getLastAlt()) + " m (" + (int) (alt * 3.2808399) + "ft) [Altstdev:" + df4.format(altStats.getStdDev()) + " m]");


                            frame.repaint();


                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        } catch (ArrayIndexOutOfBoundsException e) {
                            e.printStackTrace();
                        }


                    }
                    if (split[0].equals("TMP")) {


                        try {
                            double temp = Double.parseDouble(split[1]) / 10.0;
                            tempText.setText("Temp: " + df.format(temp) + " degC");


                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }


                    }

                    if (split[0].equals("BAT")) {


                        try {
                            int bat = Integer.parseInt(split[1], 16);
                            double volts = bat / 1000.0;
                            batText.setText("BAT: " + df2.format(volts) + " V");
                            if (out != null) {
                                //  out.println(Calendar.getInstance().getTime() + "," + System.currentTimeMillis() + "," + volts + "," + altStats.getMean() + "," + altStats.getStdDev());
                            }


                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }


                    }

                    if (split[0].equals("BST")) {
                        hardwareSettingPanel.getHardwareParameters().updateHardwareSettingsKeys(lineRead);

                    }

                    if (split[0].equals("SET")) {
                        hardwareSettingPanel.getHardwareParameters().updateHardwareSettingsValues(lineRead);

                    }

                    if (split[0].equals("BFV")) {
                        if (split.length > 2) {
                            String version = split[1] + "." + split[2];
                            hardwareVersion.setText(hardwareVersionText + version);
                            hardwareSettingPanel.updateHardwareVersion();
                        } else {
                            String version = split[1];
                            hardwareVersion.setText(hardwareVersionText + version);
                            hardwareSettingPanel.updateHardwareVersion();
                        }

                    }


                }


            }
        } catch (IOException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    public void windowOpened(WindowEvent e) {

    }

    public void windowClosing(WindowEvent e) {
        File f = new File(propertiesFileName);
        try {
            OutputStream out = new FileOutputStream(f);
            properties.store(out, "BlueFlyVario Properties version 1.0");
        } catch (IOException e1) {
            e1.printStackTrace();
        }

    }

    public void windowClosed(WindowEvent e) {

    }

    public void windowIconified(WindowEvent e) {

    }

    public void windowDeiconified(WindowEvent e) {

    }

    public void windowActivated(WindowEvent e) {

    }

    public void windowDeactivated(WindowEvent e) {

    }

    public void popupMenuWillBecomeVisible(PopupMenuEvent e) {

        String[] ports = SerialPortList.getPortNames();
        serialPorts.removeAllItems();
        for (int i = 0; i < ports.length; i++) {
            String port = ports[i];
            serialPorts.addItem(port);

        }

    }

    public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {

    }

    public void popupMenuCanceled(PopupMenuEvent e) {

    }

    public static void main(String[] args) {
        BFVDesktop gui = new BFVDesktop();

        frame = new JFrame("BFVDesktop 0.61");

        frame.add(gui);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.addWindowListener(gui);
        frame.setVisible(true);


    }

    public static byte[] hexStringToByteArray(String s) {
        int len = s.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i + 1), 16));
        }
        return data;
    }


    public void PMTKParser(String line){

        String[] mainSplit = line.split("\\*");
        String[] split = mainSplit[0].split(",");


//        printWriter.print("Some String");
//        printWriter.printf("Product name is %s and its price is %d $", "iPhone", 1000);
//        printWriter.close();

        if(split[0].equals("$PMTKLOX")){
            if(split[1].equals("0")){
                try {
                    printWriterMTK = new PrintWriter(new FileWriter("locus_record.mtk"));
                    printWriterCSV = new PrintWriter(new FileWriter("locus_record.csv"));

                    printWriterMTK.println(line);
                }
                catch (IOException e){

                }

            }
            if(split[1].equals("1")){
                int fourByteNum = 0;
                String record = "";
                for(int b = 3; b < split.length; b++){
                    record = record + split[b];
                    fourByteNum++;
                    if(fourByteNum == 4){
                        fourByteNum = 0;
                        byte[] bytes = hexStringToByteArray(record);

                        byte[] timeBytes = new byte[4];
                        timeBytes[0] = bytes[0];
                        timeBytes[1] = bytes[1];
                        timeBytes[2] = bytes[2];
                        timeBytes[3] = bytes[3];
                        int timeStamp = ByteBuffer.wrap(timeBytes).order(ByteOrder.LITTLE_ENDIAN).getInt();

//                        int timeStamp = ((bytes[3] & 0xFF) << 24)|((bytes[2] & 0xFF) << 16)|((bytes[1] & 0xFF) << 8 )|((bytes[0] & 0xFF) << 0 );
                        Date time = new java.util.Date((long)timeStamp*1000);

                        int fix = bytes[4];

                        byte[] latBytes = new byte[4];
                        latBytes[0] = bytes[5];
                        latBytes[1] = bytes[6];
                        latBytes[2] = bytes[7];
                        latBytes[3] = bytes[8];
                        float latitude = ByteBuffer.wrap(latBytes).order(ByteOrder.LITTLE_ENDIAN).getFloat();

                        byte[] longBytes = new byte[4];
                        longBytes[0] = bytes[9];
                        longBytes[1] = bytes[10];
                        longBytes[2] = bytes[11];
                        longBytes[3] = bytes[12];
                        float longitude = ByteBuffer.wrap(longBytes).order(ByteOrder.LITTLE_ENDIAN).getFloat();

                        byte[] heightBytes = new byte[4];
                        heightBytes[0] = bytes[13];
                        heightBytes[1] = bytes[14];
                        heightBytes[2] = 0;
                        heightBytes[3] = 0;
                       int height = ByteBuffer.wrap(heightBytes).order(ByteOrder.LITTLE_ENDIAN).getInt();

                       if(fix == 2){
                           SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-DD'T'HH:mm:ss'Z'");
                           dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
                           String CSVLine = new String( timeStamp + "," + dateFormat.format(time) + "," + fix +  "," + latitude + "," + longitude + "," + height);
                           textStream.print(CSVLine +"\r\n");
                           printWriterCSV.println(CSVLine);
                       }
                       record = "";
                       printWriterMTK.println(line);
                    }
                }

            }

            if(split[1].equals("2")){
                printWriterMTK.println(line);
                printWriterMTK.close();
                printWriterCSV.close();

            }

        }






    }

    public void itemStateChanged(ItemEvent e) {

            if (e.getStateChange() == ItemEvent.SELECTED) {
                Object item = e.getItem();
                if(item instanceof HardwareCommand){
                   hardwareCommandParams();
                }
                // do something with object
            }

    }
}
