package bfv.jssc;

import jssc.SerialPort;
import jssc.SerialPortException;

import java.io.IOException;
import java.io.OutputStream;

/**
 * Class that wraps a {@link jssc.SerialPort} to provide
 * {@link OutputStream} functionality.
 * 
 * @author Charles Hache <chalz@member.fsf.org>
 *
 */
public class SerialOutputStream extends OutputStream {

	SerialPort serialPort;

	/** Instantiates a SerialOutputStream for the given {@link SerialPort}
	 * @param sp The serial port to stream.
	 */
	public SerialOutputStream(SerialPort sp) {
		serialPort = sp;
	}

	@Override
	public void write(int b) throws IOException {
		try {
			serialPort.writeInt(b);
		} catch (SerialPortException e) {
			throw new IOException(e);
		}
	}

	@Override
	public void write(byte[] b) throws IOException {
		write(b, 0, b.length);

	}

	@Override
	public void write(byte[] b, int off, int len) throws IOException {
		byte[] buffer = new byte[len];
		System.arraycopy(b, off, buffer, 0, len);
		try {
            //System.out.println(bytesToHex(buffer));
			serialPort.writeBytes(buffer);
		} catch (SerialPortException e) {
			throw new IOException(e);
		}
	}

    public static String bytesToHex(byte[] bytes) {
        char[] hexArray = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
        char[] hexChars = new char[bytes.length * 3];
        int v;
        for ( int j = 0; j < bytes.length; j++ ) {
            v = bytes[j] & 0xFF;
            hexChars[j * 2] = hexArray[v >>> 4];
            hexChars[j * 2 + 1] = hexArray[v & 0x0F];
            hexChars[j * 2 + 2] = ' ';
        }
        return new String(hexChars);
    }
}