package circle;
import java.lang.*;
import java.util.ArrayList;
import java.io.*;


/**
 * Note that Java console applications need to be run through the java runtime
 */
public class Taubin
{
    /**
     * This is the main entry point for the application
     */
    public static void main(String args[]) 
    {
        try{
            BufferedReader in = new BufferedReader(new FileReader("in.txt"));
            ArrayList<Point> points = new ArrayList();
            String line;
            while((line = in.readLine()) != null){
                String[] split = line.split(",");
                Point p = new Point(Double.parseDouble(split[0]), Double.parseDouble(split[1]));
                points.add(p);
                if(points.size() >= 100){
                    double[] ret = doTaubinFit(points);
            		System.out.println(ret[0] + "," + ret[1] + "," + ret[2]);
                    break;
                    
                    
                }
                
                
                                    
            }
            
        }
        catch(Exception e){
            System.out.println(e.toString());
        }
        
    }
    
    public static double[] doTaubinFit(ArrayList<Point> points){
       	//modified from http://www.mathworks.com/matlabcentral/fileexchange/22678-circle-fit-taubin-method/content/CircleFitByTaubin.m
        
        int  n = points.size() /2;     

		Point centroid = mean(points);//   % the centroid of the data set

		//     computing moments (note: all moments will be normed, i.e. divided by n)

		double Mxx = 0; 
        double Myy = 0; 
        double Mxy = 0; 
        double Mxz = 0; 
        double Myz = 0; 
        double Mzz = 0;
        double Xi;
        double Yi;
        double Zi;
            

		for (int i = 0; i < points.size(); i++){
            Point p = points.get(i);
            Xi = p.x - centroid.x;  //%  centering data
            Yi = p.y - centroid.y;  //%  centering data
            Zi = Xi*Xi + Yi*Yi;
            Mxy = Mxy + Xi*Yi;
            Mxx = Mxx + Xi*Xi;
            Myy = Myy + Yi*Yi;
            Mxz = Mxz + Xi*Zi;
            Myz = Myz + Yi*Zi;
            Mzz = Mzz + Zi*Zi;
		}

		Mxx = Mxx/n;
	Myy = Myy/n;
	Mxy = Mxy/n;
	Mxz = Mxz/n;
	Myz = Myz/n;
	Mzz = Mzz/n;

	//    computing the coefficients of the characteristic polynomial

	double Mz = Mxx + Myy;
	double Cov_xy = Mxx*Myy - Mxy*Mxy;
	double A3 = 4*Mz;
	double A2 = -3*Mz*Mz - Mzz;
	double A1 = Mzz*Mz + 4*Cov_xy*Mz - Mxz*Mxz - Myz*Myz - Mz*Mz*Mz;
	double A0 = Mxz*Mxz*Myy + Myz*Myz*Mxx - Mzz*Cov_xy - 2*Mxz*Myz*Mxy + Mz*Mz*Cov_xy;
	double A22 = A2 + A2;
	double A33 = A3 + A3 + A3;

    System.out.println(Mz + "," + Cov_xy + "," + A3 + "," + A2 + "," + A1 + "," + A0 + "," + A22 + "," + A33);
	double xnew = 0;
	double ynew = 1e+20;
	double epsilon = 1e-12;
	int iterMax = 20;
        
    

	//% Newton's method starting at x=0

    for(int i = 0; i < iterMax; i++){
        double yold = ynew;
        ynew = A0 + xnew*(A1 + xnew*(A2 + xnew*A3));
        if(Math.abs(ynew) > Math.abs(yold)){
           System.out.println(i +  "Newton-Taubin goes wrong direction: |ynew| > |yold| ");
           xnew = 0;
           break;
        }
        double Dy = A1 + xnew*(A22 + xnew*A33);
        double xold = xnew;
        xnew = xold - ynew/Dy;
        if(Math.abs((xnew-xold)/xnew) < epsilon){
            break;
        }
        if (i >= iterMax){
            System.out.println( i + "Newton-Taubin will not converge");
            xnew = 0;
        }
        if (xnew<0.0){
            System.out.println( i + "Newton-Taubin negative root:  x=" + xnew);
            xnew = 0;
        }
    }

//%  computing the circle parameters

    double DET = xnew*xnew - xnew*Mz + Cov_xy;
	double x = Mxz*(Myy-xnew)-Myz*Mxy/DET/2;
    double y = Myz*(Mxx-xnew)-Mxz*Mxy/DET/2;
    
     double[] ret = { x + centroid.x, y + centroid.y, Math.sqrt(x*x+y*y+Mz)};
     return ret;   
        

    }
   
    
  	public static Point mean(ArrayList<Point> points){
      	double xSum = 0;
      	double ySum = 0;
      	for(int i = 0; i < points.size(); i++){
          	Point p = points.get(i);
          	xSum+= p.x;
          	ySum+= p.y;
          
      	}
      	return new Point(xSum/points.size(), ySum / points.size());
  	}
                                  
}
 