package test;

import bfv.desktop.AltPanel;
import bfv.desktop.BFVDesktop;
import bfv.hardware.HardwareSettingPanel;
import bfv.jssc.SerialInputStream;
import bfv.jssc.SerialOutputStream;
import bfv.util.Statistics;
import bfv.util.TextAreaOutputStream;
import jssc.*;

import javax.bluetooth.*;
import javax.microedition.io.StreamConnection;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.*;
import java.text.DecimalFormat;
import java.util.Properties;

/**
 * Created by IntelliJ IDEA.
 * User: Alistair
 * Date: 27/04/2011
 * Time: 7:43:46 PM
 * To change this template use File | Settings | File Templates.
 */
public class BFVTestGUI extends BFVDesktop implements ActionListener, Runnable {




    private BFVDeviceList deviceList;

    private JLabel testGPS;
    private JLabel testComms;
    private JLabel testVar;
    private JLabel testBat;

    private Color green = Color.green;
    private Color red = Color.red;

    private int testCount = 0;




    public BFVTestGUI() {
        super();

        JPanel top = new JPanel();
        top.setLayout(new BoxLayout(top, BoxLayout.X_AXIS));
        top.setAlignmentX(0.0f);



        JButton connect = new JButton("Bluetooth Devices");
        connect.setActionCommand("connect");
        connect.addActionListener(this);
        top.add(connect);

        top.add(Box.createHorizontalGlue());

        JButton test = new JButton("Test");
        test.setActionCommand("test");
        test.addActionListener(this);
        top.add(test);
        top.add(Box.createHorizontalGlue());

        testComms = new JLabel("Comms");
        testComms.setForeground(red);
        top.add(testComms);
        top.add(Box.createHorizontalGlue());

        testGPS = new JLabel("GPS");
        testGPS.setForeground(red);
        top.add(testGPS);
        top.add(Box.createHorizontalGlue());

        testVar = new JLabel("Var");
        testVar.setForeground(red);
        top.add(testVar);
        top.add(Box.createHorizontalGlue());

        testBat = new JLabel("Bat");
        testBat.setForeground(red);
        top.add(testBat);
        top.add(Box.createHorizontalGlue());

        addComponentLeft(top);


    }

    public void deviceConnected(StreamConnection connection) {
        try {
            OutputStream outputStream = connection.openOutputStream();
            setOutputStream(outputStream);
            setOut(new PrintWriter(outputStream));
            setInStream(connection.openInputStream());
            new Thread(this).start();
            getParametersFromDevice();
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

    }


    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        if (e.getActionCommand().equals("connect")) {
            if (deviceList == null) {
                try {
                    deviceList = new BFVDeviceList(LocalDevice.getLocalDevice().getDiscoveryAgent(), getTextStream(), this);
                } catch (BluetoothStateException e1) {
                    e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            } else {
                deviceList.scrollToEnd();
                deviceList.getFrame().setVisible(true);
            }

           doTest();


        }

        if (e.getActionCommand().equals("test")) {

            doTest();

        }
    }

    private boolean getTest(){
        return (testGPS.getForeground() == green) && (testComms.getForeground() == green) && (testVar.getForeground() == green);
    }

    private void doTest(){

        testGPS.setForeground(red);
        testComms.setForeground(red);
        testVar.setForeground(red);
        testBat.setForeground(red);
        testCount = 0;
        out.write("$BST*\n");
        out.flush();

    }


    public static void main(String[] args) {
        BFVTestGUI gui = new BFVTestGUI();

        frame = new JFrame("BFVTest");

        frame.add(gui);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.addWindowListener(gui);
        frame.setVisible(true);
    }

    public void run() {

        BufferedReader reader = new BufferedReader(new InputStreamReader(inStream));

        String lineRead = null;


        double qnh = Double.parseDouble(properties.getProperty("qnh"));


        DecimalFormat df1 = new DecimalFormat("0.0");
        DecimalFormat df = new DecimalFormat("+0.0;-0.0");
        DecimalFormat df2 = new DecimalFormat("0.00");

        DecimalFormat df4 = new DecimalFormat("0.0000");




        PrintStream out = null;


        try {
            out = new PrintStream(new FileOutputStream("out.csv"));
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }


        long time = 0;

        double pressureDiff = 0.0;


        NmeaChecksum nmeaChecksum = new NmeaChecksum();

        try {


            while ((lineRead = reader.readLine()) != null) {
                String[] split = lineRead.split(" ");

                testCount++;



                if (append.isSelected()) {
                    textStream.print(lineRead);

                    if(lineRead.startsWith("$")){
                        nmeaChecksum.init(lineRead);
                        if(nmeaChecksum.checkChecksum() == 0){
                            textStream.print(" <<Checksum OK >>");
                        }
                        else{
                            textStream.print(" <<Checksum Error = ");
                            textStream.print(nmeaChecksum.calculateChecksum());
                            textStream.print(" >>");
                        }

                    }

                    textStream.print("\n");
//                    if (split.length == 0) {
//                        textStream.println(lineRead);
//                    } else if (!split[0].equals("BAT")) {
//                        textStream.println(lineRead);
//                    }
                }


                if (split.length > 0) {
                    if (split[0].equals("PRS")) {


                        try {
                            int pressure = Integer.parseInt(split[1], 16);

                            double rawAlt = 44330.0 * (1 - Math.pow((pressure / qnh), 0.190295));


                            alt = rawAlt;

                            if(!firstAlt){
                                altPanel.fillData(alt);
                                firstAlt = true;
                            }
                            else{
                                altPanel.addData(rawAlt);
                                this.addAltData(rawAlt);
                            }


                            altText.setText("Alt: " + df2.format(altPanel.getLastAlt()) + " m (" + (int) (alt * 3.2808399) + "ft) [Altstdev:" + df4.format(altStats.getStdDev()) + " m]");

                            if(altStats.getStdDev() < 0.14 && testCount > 25){
                                testVar.setForeground(green);
                            }

                            if (getTest()) {
                                printOutputLine("$BTN*");

                            }

                            frame.repaint();







                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }
                        catch(ArrayIndexOutOfBoundsException e){
                            e.printStackTrace();
                        }


                    }
                    if (split[0].equals("TMP")) {


                        try {
                            double temp = Double.parseDouble(split[1]) / 10.0;
                            tempText.setText("Temp: " + df.format(temp) + " degC");


                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }


                    }

                    if (split[0].equals("BAT")) {


                        try {
                            int bat = Integer.parseInt(split[1], 16);
                            double volts = bat / 1000.0;
                            if(volts > 3.9 ){
                                testBat.setForeground(green);
                            }
                            batText.setText("BAT: " + df2.format(volts) + " V");
                            if (out != null) {
                                //  out.println(Calendar.getInstance().getTime() + "," + System.currentTimeMillis() + "," + volts + "," + altStats.getMean() + "," + altStats.getStdDev());
                            }


                        } catch (NumberFormatException e) {
                            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        }


                    }

                    if (split[0].equals("BST")) {
                        hardwareSettingPanel.getHardwareParameters().updateHardwareSettingsKeys(lineRead);

                    }

                    if (split[0].equals("SET")) {
                        hardwareSettingPanel.getHardwareParameters().updateHardwareSettingsValues(lineRead);
                        testComms.setForeground(green);

                    }

                    if (split[0].equals("BFV")) {
                        if(split.length > 2){
                            String version = split[1] + "." + split[2];
                            hardwareVersion.setText(hardwareVersionText + version);
                            hardwareSettingPanel.updateHardwareVersion();
                        }
                        else{
                            String version = split[1];
                            hardwareVersion.setText(hardwareVersionText + version);
                            hardwareSettingPanel.updateHardwareVersion();
                        }

                    }

                    if(split[0].startsWith("$G") && testCount > 25){
                        testGPS.setForeground(green);
                    }

                    if(split[0].startsWith("Audio") && testCount > 25){
                        doTest();
                    }

                    if(split[0].startsWith("MS5611") && testCount > 25){
                        doTest();
                    }

                    if(split[0].startsWith("BlueFlyVario")){
                        doTest();
                    }


                }


            }
        } catch (IOException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }
}




